#!/usr/bin/ruby
# -*- coding:utf-8 -*-

require 'kconv'

get_i18n_messages = Proc.new do
	# Get *.cpp files for translation
	cppfiles = Dir.glob("build/ppsspp/*/*.cpp")
	cppfiles = cppfiles + Dir.glob("build/ppsspp/*/*/*.cpp")

	lines = []

	cppfiles.length.times do |i|
		file = File.new(cppfiles[i], "r")
			s = file.read.toutf8
			lines = lines + s.split("\n")
		file.close
	end

	lines = lines.uniq

	i18nctg = []
	i18nmsg = []
	c = 0
	d = 0

	lines.length.times do |i|
		# Get lines that include I18NCategory
		# // I18NCategory *err = GetI18NCategory("Error");
		if lines[i].index(" = GetI18NCategory(") != nil
			i18nctg[c] = lines[i].gsub("	", "")
			c = c + 1
			next
		end

		# Get lines that include messages for translation
		# // iCardboardScreenSize, 30, 100, gr->T("Cardboard Screen Size", "Screen Size 
		# // (in % of the viewport)"), 1, screenManager(), gr->T("% of viewport")));
		s = lines[i].split('->T("')

		if s.length >= 2
			(s.length - 1).times do |n|
				# Generate "category,	message".
				# Shortest I18NCategory is "a", longest I18NCategory is "i18n"
				ctg = s[n][-4..-1]
				msg = s[n + 1].split('")')[0]

				# Remove unnecessary characters in category
				ctg = ctg.split('(')[-1]
				ctg = ctg.split("	")[-1]
				ctg = ctg.split(" ")[-1]
				ctg = ctg.split(",")[-1]

				# Generate a message for translation
				msg = msg.sub('", "', ' = ')
				msg = msg.sub('","', ' = ')

				if msg.index('", ') != nil
					msg = msg.split('", ')[0]
				end

				if msg.index(' = ') == nil
					msg = msg.chomp + " = " + msg
				end

				i18nmsg[d] = ctg + ",	" + msg
				d = d + 1
			end
		end
	end

	# Get psp_button_names
	file = File.new("build/ppsspp/Common/KeyMap.cpp", "r")
		s = file.read.toutf8
		lines = s.split("\n")
	file.close

	c = 0

	lines.length.times do |i|
		if lines[i].index("psp_button_names[] = {") != nil
			c = 1
			next
		end

		lines[i] = lines[i].sub("\" },", "\"},")

		if c == 1 && lines[i].chomp[-3..-1] == "\"},"
			i18nmsg = i18nmsg + ["mc,	" + lines[i].split('"')[-2] + " = " + \
			lines[i].split('"')[-2]]
		elsif c == 1 && lines[i].chomp == "};"
			break
		end
	end

	# Check GameSettingsScreen.cpp
	file = File.new("build/ppsspp/UI/GameSettingsScreen.cpp", "r")
		s = file.read.toutf8
		lines = s.split("\n")
	file.close

	lines.length.times do |i|
		# Get messages
		# // static const char *screenRotation[] = {"Auto", "Landscape",
		if lines[i].index("[] = {") != nil
			msg = lines[i].split('"')[1..-2]

			c = 0

			msg.length.times do |d|
				if c == 1
					msg[d] = nil
					c = 0
				else
					c = 1
				end
			end

			msg = msg.compact

			# Get long category (e.g. "screenRotation")
			# // static const char *screenRotation[] = {"Auto", "Landscape",
			ctg = lines[i].split("[] = {")[0]
			ctg = ctg.split("char *")[1]
			ctg2 = ctg

			lines.length.times do |n|
				# Convert long category to short category
				# e.g. "screenRotation" to "co"
				# // (screenRotation), co->GetName()
				if lines[n].index("(" + ctg + "), ") != nil
					ctg2 = lines[n].split("(" + ctg + "), ")[1]
					ctg2 = ctg2.split("->")[0]
				end
			end

			# Change unknown category to "sy"
			if ctg2 == ctg
				ctg2 = "sy"
			end

			ctg = ctg2

			msg.length.times do |n|
				i18nmsg = i18nmsg + [ctg + ",	" + msg[n] + " = " + msg[n]]
			end
		end
	end

	# Generate english.ini for translation
	i18nctg = i18nctg.uniq.sort
	i18nmsg = i18nmsg.uniq.sort

	file = File.new("english.ini", "w")

	i18nmsg.length.times do |i|
		# Split short category (e.g. "gr")
		if i18nmsg[i].split(",	")[0] != i18nmsg[i - 1].split(",	")[0]
			ctg = i18nmsg[i].split(",	")[0]

			i18nctg.length.times do |c|
				# Get long category (e.g. "Graphics")
				# // I18NCategory *gr = GetI18NCategory("Graphics");
				if i18nctg[c].index(" *" + ctg + " = ") != nil
					ctg = i18nctg[c].split('"')[-2]

					if i != 0
						file.puts
					end

					# Write long category (e.g. [Graphics])
					file.puts "[" + ctg + "]"
				end
			end
		end

		# Write a message for translation
		file.puts i18nmsg[i].split(",	")[1]
	end

	file.close

	# Update ini files
	inifiles = Dir.glob("build/ppsspp/assets/lang/*.ini")

	inifiles.length.times do |i|
		file = File.new("english.ini", "r")
			newmsg = file.readlines
		file.close

		file = File.new(inifiles[i], "r")
			oldmsg = file.readlines
		file.close

		newmsg.length.times do |c|
			oldmsg.length.times do |d|
				if newmsg[c].index(" = ") != nil && 
				newmsg[c].split(" = ")[0] == oldmsg[d].split(" = ")[0]
					newmsg[c] = newmsg[c].split(" = ")[0] + " = " + \
					oldmsg[d].split(" = ")[1..-1].join(" = ")
					break
				end
			end
		end

		file = File.new(inifiles[i] + ".new", "w")
			file.puts newmsg
		file.close
	end
end


# --------------------
get_i18n_messages.call
